#! /usr/bin/perl
#  Copyright (C) 2012 Jihuan Tian <tianjh04@gmail.com> and Ling Zhang
#  <zhangling06@mails.tsinghua.edu.cn>
#   
#  This file is part of SPCNDG.
#
#  SPCNDG is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SPCNDG is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SPCNDG.  If not, see <http://www.gnu.org/licenses/>.

use warnings;
use strict;

use File::Glob;
use File::Basename;

our $recur_level = 0;

RecurDirs("./src");

sub RecurDirs {
    $recur_level++;
    my $path = shift;
    gendeps($path);
    my @files = <$path/*>;
    my $file;
    for $file (@files) {
	if (-d $file) {
	    RecurDirs("$file");
	}
    }
    $recur_level--;
}

sub gendeps {
    my $path = shift;
    my $makefile = $path . "/Makefile.am";
    my @inc_dirs;

    if (-e $path . "/makedeps") {
    	unlink $path . "/makedeps";
    }
    open MAKEDEPS, '>', $path . '/makedeps' or die "Can't open makedeps file!";

    if (-e $makefile) {
	open MK, $makefile;

	my $inc_str = "";
	# extract the AM_FCFLAGS global settings
	while(<MK>) {
	    chomp;
	    if (m/AM_FCFLAGS/) {
		$inc_str = $_;
		last;
	    }
	}
	close MK;

	if (length($inc_str) > 0) {
	    my @tmp_inc_dirs0 = split /=/;
            my @tmp_inc_dirs = split /\s+/, $tmp_inc_dirs0[1];
	    foreach(@tmp_inc_dirs) {
		if (m/^-I(.*)/) {
		    my $dir = $1;
		    $dir =~ s/\$\(top_builddir\)/\./g;
		    $dir =~ s/\$\(top_srcdir\)/\./g;
		    push @inc_dirs, $dir;
		}
	    }
	}
	
	# insert the current directory as the first
	unshift @inc_dirs, $path;

	# extract the fortran source files under the INCLUDES path
	my @fort_files = <$path/*.{f90,f}>;
	
	# extract the "use" in fortran source files
	my @suffixes = qw(.f90 .f);
	if (($#fort_files + 1) > 0) {
	    foreach (@fort_files) {
		my $fort_full_name = $_;
		# open the fortran source file to see if it has "use" directives
		my @use_items;
		open FORT, $fort_full_name;
		while (<FORT>) {
		    chomp;
		    if (m/^\s*use\s+(\w+)/) {
			push @use_items, "$1"; # lowercase all the letters
		    }
		}
		close FORT;

		# if the fortran source file has "use" directives, it has dependencies
		if (($#use_items + 1) > 0) {
		    # create the dep file name
		    my ($fort_file_name, $fort_path_name, $fort_suffix) = fileparse($fort_full_name, @suffixes);
		    my $dep_name = $path . "/.deps/$fort_file_name.Po";
		    my $deps_dir = $path . "/.deps";
		    
		    # remove previous dep file and create a new one
		    if (-e $dep_name) {
			unlink $dep_name;
		    }
		    
		    # find the dependent files for each "use" directives
		    my @dep_conds;	# dependencies conditions for currrent fortran source file
		    foreach (@use_items){
			my $item = $_;
			foreach(@inc_dirs) {
			    my $idir = $_;
			    # although the dependency is for .mod files, but
			    # before any compilation, there are only .f90 files
			    if (-e ($idir . "/$item.f90")) {
				# # generate .mod file name (in lowercase)
				# my $mod_name = $idir . "/\L$item\E.mod";

				# generate .o file name
				my $mod_name = $idir . "/$item\E.o";
				my $updir_str = "";
				for(my $k = 0; $k<$recur_level-1; $k++) {
				    $updir_str = $updir_str . "../";
				}

				# # replace into $(top_builddir), however, this can't be recognized in automake
				# $mod_name =~ s/^\./\$\(top_builddir\)/;
				$mod_name =~ s/^\.\/src\//$updir_str/;
				push @dep_conds, $mod_name;
			    }
			}
		    }
		    
		    if (($#dep_conds + 1) > 0) {
			# determine if .deps dir exists
			if (! -e $deps_dir) {
			    mkdir $deps_dir;
			}
			# write the dep file
			open DEP, '>', $dep_name or die "Can't create dep file $dep_name\n";
			print "*** Generate dependencies for $fort_full_name:\n$fort_file_name.o: @dep_conds\n\n";
			print DEP "$fort_file_name.o: @dep_conds";
			print MAKEDEPS "$fort_file_name.o: @dep_conds\n";
			close DEP;
		    }
		}
	    }
	} else {
	    print "There are no fortran files under $path\n";
	}
    }

    close MAKEDEPS;
}
