!!  Copyright (C) 2012 Jihuan Tian <tianjh04@gmail.com> and Ling Zhang
!!  <zhangling06@mails.tsinghua.edu.cn>
!!   
!!  This file is part of SPCNDG.
!!
!!  SPCNDG is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  (at your option) any later version.
!!
!!  SPCNDG is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU General Public License for more details.
!!
!!  You should have received a copy of the GNU General Public License
!!  along with SPCNDG.  If not, see <http://www.gnu.org/licenses/>.

module ReadFile
  use MathFunc

  implicit none
  
  type keyvalue
     character(len=20) key
     character(len=80) value
     logical valid
  end type keyvalue
contains
  function GetKeyValue(line)
    type(keyvalue) GetKeyValue
    integer eq_index
    character(*) line

    ! initialize the return value
    GetKeyValue%key = ""
    GetKeyValue%value = ""
    GetKeyValue%valid = .false.
    
    ! if blank line, do nothing
    if (len(trim(line)) == 0) then
      return
    end if

    ! remove the prefixing and trailing blanks
    line = adjustl(line)

    ! determine if the line string is a comment
    if (index(line, "#") == 0) then
       eq_index = index(line, "=")
       GetKeyValue%key = trim(line(1:eq_index - 1))
       GetKeyValue%value = trim(adjustl(line(eq_index+1:len(line))))
       GetKeyValue%valid = .true.
    end if
  end function GetKeyValue

  subroutine SetStr(str, pair)
    character(*), intent(out) :: str
    type(keyvalue) pair
    str = pair%value
  end subroutine SetStr

  subroutine SetInt(myint, pair)
    integer, intent(out) :: myint
    type(keyvalue) pair

    read (pair%value, *) myint
  end subroutine SetInt

  subroutine SetFlt(myint, pair)
    real(SP), intent(out) :: myint
    type(keyvalue) pair

    read (pair%value, *) myint
  end subroutine SetFlt

  subroutine SetDbl(myint, pair)
    real(DP), intent(out) :: myint
    type(keyvalue) pair

    read (pair%value, *) myint
  end subroutine SetDbl

  subroutine SetIntArr(myint, pair)
    integer, dimension(:), intent(out) :: myint
    integer start_index
    integer I
    integer comma_index
    character(len=100) :: buf = ""
    type(keyvalue) pair

    start_index = 1
    I = 1
    do 
       comma_index = index(pair%value(start_index:len(pair%value)), ",")
       if (comma_index == 0) then
          buf = pair%value(start_index:len(pair%value))
          read(buf, *), myint(I)
          exit
       else
          buf = pair%value(start_index:start_index + comma_index-1)
          read(buf, *), myint(I)
          start_index = comma_index + start_index
          I = I+1
       end if
    end do
  end subroutine SetIntArr

  subroutine SetFltArr(myint, pair)
    real(SP), dimension(:), intent(out) :: myint
    integer start_index
    integer I
    integer comma_index
    character(len=100) :: buf = ""
    type(keyvalue) pair

    start_index = 1
    I = 1
    do 
       comma_index = index(pair%value(start_index:len(pair%value)), ",")
       if (comma_index == 0) then
          buf = pair%value(start_index:len(pair%value))
          read(buf, *), myint(I)
          exit
       else
          buf = pair%value(start_index:start_index + comma_index-1)
          read(buf, *), myint(I)
          start_index = comma_index + start_index
          I = I+1
       end if
    end do
  end subroutine SetFltArr

  subroutine SetDblArr(myint, pair)
    real(DP), dimension(:), intent(out) :: myint
    integer start_index
    integer I
    integer comma_index
    character(len=100) :: buf = ""
    type(keyvalue) pair

    start_index = 1
    I = 1
    do 
       comma_index = index(pair%value(start_index:len(pair%value)), ",")
       if (comma_index == 0) then
          buf = pair%value(start_index:len(pair%value))
          read(buf, *), myint(I)
          exit
       else
          buf = pair%value(start_index:start_index + comma_index-1)
          read(buf, *), myint(I)
          start_index = comma_index + start_index
          I = I+1
       end if
    end do
  end subroutine SetDblArr
end module ReadFile
